"""

Shared core Selenium module

"""

from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

import logging
import json
import requests

from .setup.installer import *
from .utils import *
from .public import *


class CoreModule(Public):
    def __init__(self, username, password, tradingType, reality, headless, browserPath, timeout):
        super().__init__()

        installer.installDriver()
        options = webdriver.FirefoxOptions()
        options.headless = headless
        options.binary_location = browserPath

        self.timeout = timeout
        self.reality = reality
        self.tradingType = tradingType

        self.driver = webdriver.Firefox(firefox_options=options)

        self.driver.get("https://www.trading212.com/en/login")
        sleep(1)
        self.driver.find_element_by_id("username-real").send_keys(username)
        sleep(0.2)
        self.driver.find_element_by_id("pass-real").send_keys(password)
        sleep(0.4)
        WebDriverWait(self.driver, self.timeout * 5).until(
            expected_conditions.element_to_be_clickable((By.CLASS_NAME, "button-login"))).click()
        sleep(0.4)

        # Cookies are required to make some requests, once acquired selenium no longer required!
        # Initial cookies are acquired for the setup
        self.cookiePayload = self.getCookies()

        # Get account data
        # self.accountData = self.get("https://live.trading212.com/frontend-data/customers/entries")
        # self.initInfo = self.get("https://live.trading212.com/rest/v3/init-info")
        
        self.getCorrectView()

        # Get the cookies again for the new view
        self.cookiePayload = self.getCookies()
        

    # Gets the current cookies
    def getCookies(self):
        rawCookies = self.driver.get_cookies()
        toReturn = rawCookies[0]["name"] + \
            '=' + rawCookies[0]["value"] + ';'
        for row in rawCookies[1:]:
            toReturn = toReturn + \
                ' ' + row["name"] + '=' + row["value"] + ';'
        return toReturn

    # Route urls based off of Reality
    def rURL(self, path=""):
        if (self.reality == Reality.Real):
            return "https://live.trading212.com/" + path
        elif (self.reality == Reality.Practice):
            return "https://demo.trading212.com/" + path
        else:
            raise Exception("Invalid reality: " + self.reality)

    def getCorrectView(self):
        # Get the correct reality
        if (self.driver.current_url != self.rURL()):
            self.driver.get(self.rURL())
        try:
            force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                expected_conditions.element_to_be_clickable((By.XPATH, "/html/body/div[6]/div[3]/div[2]"))))
        except:
            logging.warn("Didn't shut the new account view")

        # Switch to correct TradingType
        try:
            force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                expected_conditions.element_to_be_clickable((By.XPATH, "/html/body/div[1]/div[1]/div[1]/div/div[2]/div[3]"))))
            if (self.tradingType == TradingType.ISA):
                force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                    expected_conditions.element_to_be_clickable((By.XPATH, "/html/body/div[1]/div[10]/div/div/div/div/div[1]/div/div[1]/div[3]"))))
            elif (self.tradingType == TradingType.Equity):
                force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                    expected_conditions.element_to_be_clickable((By.XPATH, "/html/body/div[1]/div[10]/div/div/div/div/div[1]/div/div[1]/div[2]"))))
            elif (self.tradingType == TradingType.CFD):
                force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                    expected_conditions.element_to_be_clickable((By.XPATH, "/html/body/div[1]/div[10]/div/div/div/div/div[1]/div/div[1]/div[1]"))))
            else:
                raise Exception("Invalid tradingType: " + self.tradingType)
        except:
            pass

        try:
            force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                expected_conditions.element_to_be_clickable((By.XPATH, "/html/body/div[4]/div[5]"))))
            if (self.tradingType == TradingType.ISA):
                 force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                expected_conditions.element_to_be_clickable((By.XPATH, '//*[@id="isaSwitchButton"]'))))
            elif (self.tradingType == TradingType.Equity):
                force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                    expected_conditions.element_to_be_clickable((By.XPATH, '//*[@id="equitySwitchButton"]'))))
            elif (self.tradingType == TradingType.CFD):
                force_click(WebDriverWait(self.driver, self.timeout * 2).until(
                    expected_conditions.element_to_be_clickable((By.XPATH, '//*[@id="cfdSwitchButton"]'))))
        except:
            pass

        # If tradingType is not CFD request the new web app
        if (self.tradingType != TradingType.CFD):
            # Check if already using beta
            try:
                if (self.driver.current_url[:-4] != "beta"):
                    # Get beta
                    url = self.rURL("frontend-data/customers/entries")
                    payload = "[{\"key\":\"platform-pref\",\"value\":\"new\"}]"
                    get(url=url, payload=payload, cookies=self.cookiePayload)
                    self.driver.get(self.rURL("beta"))
            except:
                pass
            # Todo handle isa or equity
        else:
            # Todo handle CFD
            1



from distutils.core import setup

setup(
    name='viaduct',         # How you named your package folder (MyLib)
    packages=['viaduct'],   # Chose the same as "name"
    version='0.0.4',      # Start with a small number and increase it with every change you make
    # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    license='gpl-3.0',
    # Give a short description about your library
    description='Viaduct is a Trading212 REST API wrapper for python',
    long_description="Viaduct is a Python REST API wrapper that utilises Trading212's REST API that normally communciates exclusively with first party apps. This has not been offically released publicly but can be reverse engineered. Selenium is used to scrape required cookies and customer data from the web app but is no longer required after this point. REST network calls are used enabling greater functionallity and speed than pure web scraping. The use of a wrapper ensures any API changes in the future will not impact pre-existing Trading212 dependeant programs as the wrapper itself can be updated instead of the underlying code.",
    author='Harry Twigg',                   # Type in your name
    author_email='harrytwigg111@gmail.com',      # Type in your E-Mail
    # Provide either the link to your github or to your website
    url='https://github.com/harrytwigg/Viaduct-Trading212-Python-API',
    # I explain this later on
    download_url='https://github.com/harrytwigg/Viaduct-Trading212-Python-API/archive/0.0.4.tar.gz',
    keywords=['python', 'api', 'rest', 'api-wrapper', 'viaduct', 'trading212',
              'trading212-api'],   # Keywords that define your package best
    install_requires=[            # I get to this in a second
        'validators',
        'beautifulsoup4',
        'selenium',
        'matplotlib',
        'selenium',
        'requests',
    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 3 - Alpha',
        # Define that your audience are developers
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',   # Again, pick a license
        # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
